@@ -55,6 +55,11 @@ int DBusMenuExporterPrivate::idForAction(QAction *action) const
 
 void DBusMenuExporterPrivate::addMenu(QMenu *menu, int parentId)
 {
+    if (menu->findChild<DBusMenu *>()) {
+        // This can happen if a menu is removed from its parent and added back
+        // See KDE bug 254066
+        return;
+    }
     new DBusMenu(menu, q, parentId);
     Q_FOREACH(QAction *action, menu->actions()) {
         addAction(action, parentId);
@@ -138,10 +143,10 @@ QMenu *DBusMenuExporterPrivate::menuForId(int id) const
         return m_rootMenu;
     }
     QAction *action = m_actionForId.value(id);
-    DMRETURN_VALUE_IF_FAIL(action, 0);
-    QMenu *menu = action->menu();
-    DMRETURN_VALUE_IF_FAIL(menu, 0);
-    return menu;
+    // Action may not be in m_actionForId if it has been deleted between the
+    // time it was announced by the exporter and the time the importer asks for
+    // it.
+    return action ? action->menu() : 0;
 }
 
 void DBusMenuExporterPrivate::writeXmlForMenu(QXmlStreamWriter *writer, QMenu *menu, int id)
@@ -178,8 +183,13 @@ void DBusMenuExporterPrivate::updateAction(QAction *action)
 
 void DBusMenuExporterPrivate::addAction(QAction *action, int parentId)
 {
+    int id = m_idForAction.value(action, -1);
+    if (id != -1) {
+        DMWARNING << "Already tracking action" << action->text() << "under id" << id;
+        return;
+    }
     QVariantMap map = propertiesForAction(action);
-    int id = m_nextId++;
+    id = m_nextId++;
     QObject::connect(action, SIGNAL(destroyed(QObject*)), q, SLOT(slotActionDestroyed(QObject*)));
     m_actionForId.insert(id, action);
     m_idForAction.insert(action, id);
@@ -282,7 +292,7 @@ void DBusMenuExporter::doUpdateActions()
         }
         d->m_actionProperties[action] = d->propertiesForAction(action);
         QMenu *menu = action->menu();
-        if (menu && !menu->findChild<DBusMenu *>()) {
+        if (menu) {
             d->addMenu(menu, id);
         }
         d->m_dbusObject->ItemUpdated(id);
