@@ -1,4 +1,4 @@
-/* This file is part of the KDE libraries
+/* This file is part of the dbusmenu-qt library
    Copyright 2009 Canonical
    Author: Aurelien Gateau <aurelien.gateau@canonical.com>
 
@@ -247,7 +247,7 @@ void DBusMenuExporterTest::testDynamicSubMenu()
     // too often because it causes refreshes
     QDBusInterface iface(TEST_SERVICE, TEST_OBJECT_PATH);
     ManualSignalSpy layoutUpdatedSpy;
-    QDBusConnection::sessionBus().connect(TEST_SERVICE, TEST_OBJECT_PATH, "org.ayatana.dbusmenu", "LayoutUpdated", "ui", &layoutUpdatedSpy, SLOT(receiveCall(uint, int)));
+    QDBusConnection::sessionBus().connect(TEST_SERVICE, TEST_OBJECT_PATH, "com.canonical.dbusmenu", "LayoutUpdated", "ui", &layoutUpdatedSpy, SLOT(receiveCall(uint, int)));
 
     // Create our test menu
     QMenu inputMenu;
@@ -338,7 +338,7 @@ void DBusMenuExporterTest::testRadioItems()
 
     // Click a2
     ManualSignalSpy spy;
-    QDBusConnection::sessionBus().connect(TEST_SERVICE, TEST_OBJECT_PATH, "org.ayatana.dbusmenu", "ItemUpdated", &spy, SLOT(receiveCall(int)));
+    QDBusConnection::sessionBus().connect(TEST_SERVICE, TEST_OBJECT_PATH, "com.canonical.dbusmenu", "ItemUpdated", &spy, SLOT(receiveCall(int)));
     QVariant empty = QVariant::fromValue(QDBusVariant(QString()));
     uint timestamp = QDateTime::currentDateTime().toTime_t();
     iface.call("Event", a2Id, "clicked", empty, timestamp);
@@ -562,7 +562,7 @@ void DBusMenuExporterTest::testActivateAction()
     QVERIFY2(iface.isValid(), qPrintable(iface.lastError().message()));
 
     ManualSignalSpy spy;
-    QDBusConnection::sessionBus().connect(TEST_SERVICE, TEST_OBJECT_PATH, "org.ayatana.dbusmenu", "ItemActivationRequested", "iu", &spy, SLOT(receiveCall(int, uint)));
+    QDBusConnection::sessionBus().connect(TEST_SERVICE, TEST_OBJECT_PATH, "com.canonical.dbusmenu", "ItemActivationRequested", "iu", &spy, SLOT(receiveCall(int, uint)));
 
     // Get item ids
     QDBusReply<DBusMenuItemList> reply = iface.call("GetChildren", 0, QStringList());
@@ -585,4 +585,41 @@ void DBusMenuExporterTest::testActivateAction()
     QCOMPARE(spy.takeFirst().at(0).toInt(), id2);
 }
 
+static int trackCount(QMenu* menu)
+{
+    QList<QObject*> lst = menu->findChildren<QObject*>();
+    int count = 0;
+    Q_FOREACH(QObject* child, lst) {
+        if (qstrcmp(child->metaObject()->className(), "DBusMenu") == 0) {
+            ++count;
+        }
+    }
+    return count;
+}
+
+// Check we do not create more than one DBusMenu object for each menu
+// See KDE bug 254066
+void DBusMenuExporterTest::testTrackActionsOnlyOnce()
+{
+    // Create a menu with a submenu, unplug the submenu and plug it back. The
+    // submenu should not have more than one DBusMenu child object.
+    QMenu mainMenu;
+    QVERIFY(QDBusConnection::sessionBus().registerService(TEST_SERVICE));
+    DBusMenuExporter *exporter = new DBusMenuExporter(TEST_OBJECT_PATH, &mainMenu);
+
+    QMenu* subMenu = new QMenu("File");
+    subMenu->addAction("a1");
+    mainMenu.addAction(subMenu->menuAction());
+
+    QTest::qWait(500);
+    QCOMPARE(trackCount(subMenu), 1);
+
+    mainMenu.removeAction(subMenu->menuAction());
+
+    mainMenu.addAction(subMenu->menuAction());
+
+    QTest::qWait(500);
+    QCOMPARE(trackCount(subMenu), 1);
+}
+
 #include "dbusmenuexportertest.moc"
